import requests
import xmltodict


class Modem():
    """
    Huawei modem access over http
    Oliver Archner 
    """

    def __init__(self, ip):
        self.ip = ip
        self.sessionID = None
        self.token = None

    def open(self):
        try:
            r = requests.get(url='http://' + self.ip + '/api/webserver/SesTokInfo',
                             allow_redirects=False, timeout=(2.0, 2.0))
        except requests.exceptions.RequestException as e:
            return False
        try:
            d = xmltodict.parse(r.text, xml_attribs=True)
            if 'response' in d and 'SesInfo' in d['response']:
                self.sessionID = d['response']['SesInfo']
            if 'response' in d and 'TokInfo' in d['response']:
                self.token = d['response']['TokInfo']
        except:
            return False
        return True

    def close(self):
        self.sessionID = None
        self.token = None

    def fetch(self, resource, body=""):
        if self.sessionID is None or self.token is None:
            raise Exception('Not connected.')

        url = 'http://' + self.ip + resource

        if body == "":
            r = requests.get(
                url, headers={'Cookie': self.sessionID}, allow_redirects=False, timeout=(2.0, 2.0))
        else:
            r = requests.post(url, data=body, headers={'Cookie': self.sessionID, '__RequestVerificationToken': self.token},
                              allow_redirects=False, timeout=(2.0, 2.0), )
        if r.status_code == 200:
            d = xmltodict.parse(r.text, xml_attribs=True)
            if 'error' in d:
                raise Exception('Received error code ' + d['error']['code'])
            return d
        else:
            raise Exception('Received status code ' + str(r.status_code))

    def getStatus(self):
        return self.fetch('/api/monitoring/status')

    def reboot(self):
        return self.fetch('/api/device/control', '<request><Control>1</Control></request>')

    def getTrafficStatistics(self):
        return self.fetch('/api/monitoring/traffic-statistics')
